<?php

/**
 * @file
 * Discource SSO endpoint functionality.
 */

/**
 * Menu callback function used for discourse/sso.
 */
function discourse_sso_endpoint() {
  // Get SSO input parameters.
  // They have been already validated in access callback.
  $payload = discourse_sso_get_payload();
  $nonce = discourse_sso_get_nonce($payload);

  discourse_sso_process_response($nonce);
}

/**
 * Process response to discourse site.
 *
 * @param string $nonce
 *   In security engineering, a nonce is an arbitrary number used only once
 *   in a cryptographic communication. It ensures that old communications
 *   cannot be reused in replay attacks.
 */
function discourse_sso_process_response($nonce) {
  // Prepare response data.
  $user = discourse_sso_authenticate_user();
  $payload = discourse_sso_create_payload($user, $nonce);
  $sig = discourse_sso_sign_payload($payload);

  // Response to the discourse forum.
  discourse_sso_response($payload, $sig);
}

/**
 * Authenticate user.
 *
 * @global object $user
 *
 * @return object
 *   User object with user information.
 */
function discourse_sso_authenticate_user() {
  global $user;

  // User authentication.
  if (!user_is_logged_in()) {
    // Redirect user for login page and set redirection/destination back
    // to here after the user is logged in.
    drupal_goto('user', array('query' => drupal_get_destination()));
  }

  return $user;
}

/**
 * Create payload.
 *
 * Payload contains nonce + user information.
 *
 * @param object $user
 *   User object with user information.
 * @param string $nonce
 *   In security engineering, a nonce is an arbitrary number used only once
 *   in a cryptographic communication. It ensures that old communications
 *   cannot be reused in replay attacks.
 *
 * @return string
 *   A payload. It's a base64 encoded query of parameters.
 */
function discourse_sso_create_payload($user, $nonce) {
  $user_params = array(
    "nonce" => $nonce,
    "external_id" => $user->name,
    "email" => $user->mail,
    "username" => $user->name,
  );

  $user_params += discourse_sso_get_optional_user_params($user);

  // Create payload from user params.
  $query = drupal_http_build_query($user_params);
  $payload = base64_encode($query);

  return $payload;
}

/**
 * Get optional user parameters.
 *
 * @param object $user
 *   User object with user information.
 *
 * @return array
 *   An array of optional user parameters.
 */
function discourse_sso_get_optional_user_params($user) {
  $user_params = array();

  // Load all user information and create its entity metadata wrapper.
  $user_wrapper = entity_metadata_wrapper('user', user_load($user->uid));

  // Set the user fullname if it exists.
  if (isset($user_wrapper->field_first_name) && isset($user_wrapper->field_last_name)) {
    $user_params['name'] = $user_wrapper->field_first_name->value() . ' ' . $user_wrapper->field_last_name->value();
  }

  return $user_params;
}

/**
 * Sign payload.
 *
 * @param string $payload
 *   Base64 encoded data.
 *
 * @return string
 *   Returns a payload signature.
 */
function discourse_sso_sign_payload($payload) {
  $sso_secret = discourse_sso_get_secret();

  return hash_hmac("sha256", $payload, $sso_secret);
}

/**
 * Response to the discourse site.
 *
 * @param string $payload
 *   Base64 encoded data.
 * @param string $sig
 *   Payload signature.
 */
function discourse_sso_response($payload, $sig) {
  $query = array("sso" => $payload, "sig" => $sig);
  $sso_login_url = discourse_sso_get_url() . '/session/sso_login';

  drupal_goto($sso_login_url, array('query' => $query, 'external' => TRUE));
}
